@@ -1,8 +1,8 @@
 /*
  *  Hamlib CI-V backend - description of IC-275 and variations
- *  Copyright (c) 2000-2002 by Stephane Fillod
+ *  Copyright (c) 2000-2008 by Stephane Fillod
  *
- *	$Id: ic275.c,v 1.4 2002/11/04 22:40:54 fillods Exp $
+ *	$Id: ic275.c,v 1.9 2008/10/31 07:49:11 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -24,7 +24,9 @@
 #include "config.h"
 #endif
 
-#include <hamlib/rig.h>
+#include <stdlib.h>
+
+#include "hamlib/rig.h"
 #include "icom.h"
 
 
@@ -34,7 +36,6 @@
 
 #define IC275_VFO_OPS (RIG_OP_FROM_VFO|RIG_OP_TO_VFO)
 
-#define IC275_STR_CAL { 0, { } }
 
 /*
  * FIXME: this appears to be the IC-275A/E
@@ -43,17 +44,16 @@
 static const struct icom_priv_caps ic275_priv_caps = { 
 		0x10,	/* default address */
 		0,		/* 731 mode */
-		ic737_ts_sc_list,
-		IC275_STR_CAL
+		ic737_ts_sc_list
 };
 
 const struct rig_caps ic275_caps = {
 .rig_model =  RIG_MODEL_IC275,
 .model_name = "IC-275", 
 .mfg_name =  "Icom", 
-.version =  "0.2", 
+.version =  BACKEND_VER ".1",
 .copyright =  "LGPL",
-.status =  RIG_STATUS_ALPHA,
+.status =  RIG_STATUS_BETA,
 .rig_type =  RIG_TYPE_TRANSCEIVER,
 .ptt_type =  RIG_PTT_NONE,
 .dcd_type =  RIG_DCD_NONE,
@@ -91,9 +91,9 @@ const struct rig_caps ic275_caps = {
 .chan_desc_sz =  0,
 
 .chan_list =  {
-		   {   1,  99, RIG_MTYPE_MEM  },
-		   { 100, 101, RIG_MTYPE_EDGE },
-		   { 102, 102, RIG_MTYPE_CALL },
+		   {   1,  99, RIG_MTYPE_MEM, IC_MIN_MEM_CAP  },
+		   { 100, 101, RIG_MTYPE_EDGE, IC_MIN_MEM_CAP },
+		   { 102, 102, RIG_MTYPE_CALL, IC_MIN_MEM_CAP },
 		   RIG_CHAN_END,
 		},
 
