@@ -1,8 +1,8 @@
 /*
  *  Hamlib CI-V backend - description of IC-775 and variations
- *  Copyright (c) 2000-2002 by Stephane Fillod
+ *  Copyright (c) 2000-2008 by Stephane Fillod
  *
- *	$Id: ic775.c,v 1.6 2005/04/03 19:53:51 fillods Exp $
+ *	$Id: ic775.c,v 1.7 2008/10/25 14:00:52 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -28,7 +28,7 @@
 
 #include <hamlib/rig.h>
 #include "icom.h"
-
+#include "bandplan.h"
 
 #define IC775_ALL_RX_MODES (RIG_MODE_AM|RIG_MODE_CW|RIG_MODE_SSB|RIG_MODE_RTTY|RIG_MODE_FM)
 #define IC775_1MHZ_TS_MODES (RIG_MODE_AM|RIG_MODE_FM)
@@ -44,7 +44,7 @@
 
 #define IC775_VFO_OPS (RIG_OP_FROM_VFO|RIG_OP_TO_VFO)
 
-#define IC775_STR_CAL { 0, { } }
+#define IC775_ANTS RIG_ANT_1
 
 /*
  */
@@ -58,7 +58,7 @@ const struct rig_caps ic775_caps = {
 .rig_model =  RIG_MODEL_IC775,
 .model_name = "IC-775", 
 .mfg_name =  "Icom", 
-.version =  BACKEND_VER, 
+.version =  BACKEND_VER ".1",
 .copyright =  "LGPL",
 .status =  RIG_STATUS_UNTESTED,
 .rig_type =  RIG_TYPE_TRANSCEIVER,
@@ -66,7 +66,7 @@ const struct rig_caps ic775_caps = {
 .dcd_type =  RIG_DCD_NONE,
 .port_type =  RIG_PORT_SERIAL,
 .serial_rate_min =  1200,
-.serial_rate_max =  1200,
+.serial_rate_max =  19200,
 .serial_data_bits =  8,
 .serial_stop_bits =  1,
 .serial_parity =  RIG_PARITY_NONE,
@@ -98,13 +98,17 @@ const struct rig_caps ic775_caps = {
 .chan_desc_sz =  0,
 
 .chan_list =  {
-		   {   1,  10, RIG_MTYPE_MEM  },
-		   {  11,  12, RIG_MTYPE_EDGE },
+		   {   1,  10, RIG_MTYPE_MEM,  IC_MIN_MEM_CAP },
+		   {  11,  12, RIG_MTYPE_EDGE, IC_MIN_MEM_CAP },
 		   RIG_CHAN_END,
 		},
 
-.rx_range_list1 =   { RIG_FRNG_END, },	/* FIXME: enter region 1 setting */
-.tx_range_list1 =   { RIG_FRNG_END, },
+.rx_range_list1 =   { {kHz(30),MHz(30),IC775_ALL_RX_MODES,-1,-1,IC775_VFO_ALL},
+	RIG_FRNG_END, },
+.tx_range_list1 =   {
+	FRQ_RNG_HF(1,IC775_OTHER_TX_MODES, W(10),W(100),IC775_VFO_ALL,IC775_ANTS),
+	FRQ_RNG_HF(1,IC775_AM_TX_MODES, W(10),W(40),IC775_VFO_ALL,IC775_ANTS),   /* AM class */
+	RIG_FRNG_END, },
 
 .rx_range_list2 =   { {kHz(30),MHz(30),IC775_ALL_RX_MODES,-1,-1,IC775_VFO_ALL},
 	RIG_FRNG_END, },
