@@ -3,7 +3,7 @@
  *  Contributed by Francois Retief <fgretief@sun.ac.za>
  *  Copyright (c) 2000-2004 by Stephane Fillod
  *
- *      $Id: ic910.c,v 1.13 2007/12/13 18:48:56 y32kn Exp $
+ *      $Id: ic910.c,v 1.15 2008/11/02 16:14:57 y32kn Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -235,7 +235,7 @@ const struct rig_caps ic910_caps = {
 .mfg_name =  "Icom",
 .version =  BACKEND_VER,
 .copyright =  "LGPL",
-.status =  RIG_STATUS_ALPHA,
+.status =  RIG_STATUS_BETA,
 .rig_type =  RIG_TYPE_TRANSCEIVER,
 .ptt_type =  RIG_PTT_NONE,
 .dcd_type =  RIG_DCD_RIG,
@@ -314,15 +314,10 @@ const struct rig_caps ic910_caps = {
     RIG_TS_END, },
     /* mode/filter list, remember: order matters! */
 .filters =     {
-    {RIG_MODE_CW, kHz(2.3)},   /* builtin */
-    {RIG_MODE_CW, Hz(600)},   /* builtin */
-    {RIG_MODE_CW, kHz(2.3)},   /* builtin */
-    {RIG_MODE_SSB, kHz(2.3)},
-    {RIG_MODE_SSB, kHz(2.3)},
-    {RIG_MODE_SSB, kHz(2.3)},
-    {RIG_MODE_FM, kHz(15)},                 /* builtin */
-    {RIG_MODE_FM, kHz(6)},                 /* builtin */
-    {RIG_MODE_FM, kHz(15)},                 /* builtin */
+    {RIG_MODE_CW | RIG_MODE_SSB, kHz(2.3)},     /* builtin */
+    {RIG_MODE_CW, Hz(600)},                     /* with optional FL-132/Fl133 CW filters */
+    {RIG_MODE_FM, kHz(15)},                     /* builtin */
+    {RIG_MODE_FM, kHz(6)},                      /* builtin */
      RIG_FLT_END, },
 .str_cal = IC910_STR_CAL,
 
