@@ -1,8 +1,9 @@
 /*
  *  Hamlib Interface - list of known rotators
+ *  Copyright (c) 2000-2008 by Stephane Fillod
  *  Copyright (c) 2000-2002 by Stephane Fillod and Frank Singleton
  *
- *	$Id: rotlist.h,v 1.9 2003/06/22 19:38:26 fillods Exp $
+ *	$Id: rotlist.h,v 1.14 2008/10/31 07:45:17 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -26,8 +27,12 @@
 #define ROT_MAKE_MODEL(a,b) ((a)*100+(b))
 #define ROT_BACKEND_NUM(a) ((a)/100)
 
+/**
+ * \addtogroup rotator
+ * @{
+ */
+
 /*! \file rotlist.h
- *  \ingroup rot
  *  \brief Hamlib rotator model definitions.
  *
  *  This file contains rotator model definitions for the Hamlib rotator API.
@@ -42,7 +47,6 @@
  *  use. It is done with the rot_init() API call.
  */
 
-
 #define ROT_MODEL_NONE 0
 
 /*! \def ROT_MODEL_DUMMY
@@ -56,6 +60,7 @@
 #define ROT_DUMMY 0
 #define ROT_BACKEND_DUMMY "dummy"
 #define ROT_MODEL_DUMMY ROT_MAKE_MODEL(ROT_DUMMY, 1)
+#define ROT_MODEL_NETROTCTL ROT_MAKE_MODEL(ROT_DUMMY, 2)
 
 	/*
 	 * RPC Network pseudo-backend
@@ -71,7 +76,6 @@
 #define ROT_BACKEND_RPC "rpcrot"
 #define ROT_MODEL_RPC ROT_MAKE_MODEL(ROT_RPC, 1)
 
-
 	/*
 	 * Easycomm
 	 */
@@ -136,13 +140,30 @@
 #define ROT_BACKEND_SARTEK "sartek"
 #define ROT_MODEL_SARTEK1 ROT_MAKE_MODEL(ROT_SARTEK, 1)
 
+/*! \def ROT_MODEL_GS232A
+ *  \brief A macro that returns the model number of the GS-232A backend.
+ *
+ *  The GS-232A backend can be used with rotators that support the
+ *  GS-232A protocol.
+ */
+#define ROT_GS232A 6
+#define ROT_BACKEND_GS232A "gs232a"
+#define ROT_MODEL_GS232A ROT_MAKE_MODEL(ROT_GS232A, 1)
+
+/*! \def ROT_MODEL_PCROTOR
+ *  \brief A macro that returns the model number of the PcRotor/WA6UFQ backend.
+ *
+ *  The kit backend can be used with home brewed rotators.
+ */
+#define ROT_KIT 7
+#define ROT_BACKEND_KIT "kit"
+#define ROT_MODEL_PCROTOR ROT_MAKE_MODEL(ROT_KIT, 1)
 
 /*! \typedef typedef int rot_model_t
     \brief Convenience type definition for rotator model.
 */
 typedef int rot_model_t;
 
-
 /*! \def ROT_BACKEND_LIST
  *  \brief Static list of rotator models.
  *
@@ -157,6 +178,8 @@ typedef int rot_model_t;
         { ROT_FODTRACK, ROT_BACKEND_FODTRACK }, \
         { ROT_ROTOREZ, ROT_BACKEND_ROTOREZ }, \
         { ROT_SARTEK, ROT_BACKEND_SARTEK }, \
+        { ROT_GS232A, ROT_BACKEND_GS232A }, \
+        { ROT_KIT, ROT_BACKEND_KIT }, \
         { 0, NULL }, /* end */  \
 }
 
@@ -169,3 +192,5 @@ typedef int rot_model_t;
  */
 
 #endif /* _ROTLIST_H */
+
+/** @} */
