@@ -1,8 +1,8 @@
 /*
  *  Hamlib Interface - main file
- *  Copyright (c) 2000-2006 by Stephane Fillod and Frank Singleton
+ *  Copyright (c) 2000-2008 by Stephane Fillod and Frank Singleton
  *
- *	$Id: rig.c,v 1.95 2007/11/26 20:54:12 n0nb Exp $
+ *	$Id: rig.c,v 1.100 2008/10/31 23:08:59 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -30,7 +30,7 @@
  * \brief Ham Radio Control Libraries interface
  * \author Stephane Fillod
  * \author Frank Singleton
- * \date 2000-2006
+ * \date 2000-2008
  *
  * Hamlib provides a user-callable API, a set of "front-end" routines that
  * call rig-specific "back-end" routines whichactually communicate with 
@@ -64,6 +64,7 @@
 #include "serial.h"
 #include "parallel.h"
 #include "usb_port.h"
+#include "network.h"
 #include "event.h"
 
 /**
@@ -76,7 +77,7 @@ const char hamlib_version[] = "Hamlib version " PACKAGE_VERSION;
  * \brief Hamlib copyright notice
  */
 const char hamlib_copyright[] = 
-  "Copyright (C) 2000-2007 Stephane Fillod and Frank Singleton\n"
+  "Copyright (C) 2000-2008 Stephane Fillod and Frank Singleton\n"
   "This is free software; see the source for copying conditions.  There is NO\n"
   "warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.";
 
@@ -85,6 +86,8 @@ const char hamlib_copyright[] =
 
 #if defined(WIN32) && !defined(__CYGWIN__)
 #define DEFAULT_SERIAL_PORT "\\\\.\\COM1"
+#elif BSD
+#define DEFAULT_SERIAL_PORT "/dev/cuaa0"
 #else
 #define DEFAULT_SERIAL_PORT "/dev/ttyS0"
 #endif
@@ -293,6 +296,10 @@ RIG * HAMLIB_API rig_init(rig_model_t rig_model)
 	strncpy(rs->rigport.pathname, DEFAULT_PARALLEL_PORT, FILPATHLEN);
 	break;
 
+	case RIG_PORT_NETWORK:
+	strncpy(rs->rigport.pathname, "localhost:4532", FILPATHLEN);
+	break;
+
 	default:
 	strncpy(rs->rigport.pathname, "", FILPATHLEN);
 	}
@@ -478,8 +485,12 @@ int HAMLIB_API rig_open(RIG *rig)
 	case RIG_PORT_RPC:
 		break;	/* ez :) */
 
-	case RIG_PORT_NETWORK:	/* not implemented yet! */
-		return -RIG_ENIMPL;
+	case RIG_PORT_NETWORK:
+		status = network_open(&rs->rigport, 4532);
+		if (status < 0)
+			return status;
+		break;
+
 	default:
 		return -RIG_EINVAL;
 	}
@@ -932,6 +943,9 @@ int HAMLIB_API rig_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width
 		rig->state.current_width = *width;
 	}
 
+	if (*width == RIG_PASSBAND_NORMAL && *mode != RIG_MODE_NONE)
+		*width = rig_passband_normal (rig, *mode);
+
 	return retcode;
 }
 
@@ -954,7 +968,7 @@ pbwidth_t HAMLIB_API rig_passband_normal(RIG *rig, rmode_t mode)
 	int i;
 
 	if (!rig)
-		return 0;	/* huhu! */
+		return RIG_PASSBAND_NORMAL;	/* huhu! */
 
 	rs = &rig->state;
 
@@ -964,7 +978,7 @@ pbwidth_t HAMLIB_API rig_passband_normal(RIG *rig, rmode_t mode)
 		}
 	}
 
-	return 0;
+	return RIG_PASSBAND_NORMAL;
 }
 
 /**
@@ -1234,16 +1248,25 @@ int HAMLIB_API rig_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt)
 		break;
 
 	case RIG_PTT_SERIAL_RTS:
+		if (caps->get_ptt)
+			return caps->get_ptt(rig, vfo, ptt);
+
 		retcode = ser_get_rts(&rig->state.pttport, &status);
 		*ptt = status ? RIG_PTT_ON : RIG_PTT_OFF;
 		return retcode;
 
 	case RIG_PTT_SERIAL_DTR:
+		if (caps->get_ptt)
+			return caps->get_ptt(rig, vfo, ptt);
+
 		retcode = ser_get_dtr(&rig->state.pttport, &status);
 		*ptt = status ? RIG_PTT_ON : RIG_PTT_OFF;
 		return retcode;
 
 	case RIG_PTT_PARALLEL:
+		if (caps->get_ptt)
+			return caps->get_ptt(rig, vfo, ptt);
+
 		return par_ptt_get(&rig->state.pttport, ptt);
 
 	case RIG_PTT_NONE:
@@ -1685,7 +1708,11 @@ int HAMLIB_API rig_set_split_mode(RIG *rig, vfo_t vfo, rmode_t tx_mode, pbwidth_
  * \param tx_mode	The location where to store the current transmit split mode
  * \param tx_width	The location where to store the current transmit split width
  *
- *  Retrieves the current split(TX) mode.
+ *  Retrieves the current split(TX) mode and passband.
+ *  If the backend is unable to determine the width, the \a tx_width
+ *  will be set to RIG_PASSBAND_NORMAL as a default.
+ *  The value stored at \a tx_mode location equals RIG_MODE_NONE
+ *  when the current mode of the VFO is not defined (e.g. blank memory).
  *
  * \return RIG_OK if the operation has been sucessful, otherwise 
  * a negative value if an error occured (in which case, cause is 
@@ -1730,6 +1757,10 @@ int HAMLIB_API rig_get_split_mode(RIG *rig, vfo_t vfo, rmode_t *tx_mode, pbwidth
 	} else {
 		caps->vfo_op(rig, vfo, RIG_OP_TOGGLE);
 	}
+
+	if (*tx_width == RIG_PASSBAND_NORMAL && *tx_mode != RIG_MODE_NONE)
+		*tx_width = rig_passband_normal (rig, *tx_mode);
+
 	return retcode;
 }
 
