@@ -1,8 +1,8 @@
 /*
  *  Hamlib Interface - main file
- *  Copyright (c) 2000-2006 by Stephane Fillod and Frank Singleton
+ *  Copyright (c) 2000-2008 by Stephane Fillod and Frank Singleton
  *
- *	$Id: rotator.c,v 1.21 2006/10/15 00:27:52 aa6e Exp $
+ *	$Id: rotator.c,v 1.24 2008/09/23 22:02:39 fillods Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -29,7 +29,7 @@
  * \file src/rotator.c
  * \brief Rotator interface
  * \author Stephane Fillod
- * \date 2000-2006
+ * \date 2000-2008
  *
  * Hamlib interface is a frontend implementing rotator wrapper functions.
  */
@@ -57,6 +57,8 @@
 #include "hamlib/rotator.h"
 #include "serial.h"
 #include "parallel.h"
+#include "usb_port.h"
+#include "network.h"
 #include "rot_conf.h"
 #include "token.h"
 
@@ -65,9 +67,12 @@
 
 #if defined(WIN32) && !defined(__CYGWIN__)
 #define DEFAULT_SERIAL_PORT "\\\\.\\COM1"
+#elif BSD
+#define DEFAULT_SERIAL_PORT "/dev/cuaa0"
 #else
 #define DEFAULT_SERIAL_PORT "/dev/ttyS0"
 #endif
+
 #if defined(WIN32)
 #define DEFAULT_PARALLEL_PORT "\\\\.\\$VDMLPT1"
 #elif defined(HAVE_DEV_PPBUS_PPI_H)
@@ -228,6 +233,10 @@ ROT * HAMLIB_API rot_init(rot_model_t rot_model)
 	strncpy(rs->rotport.pathname, DEFAULT_PARALLEL_PORT, FILPATHLEN);
 	break;
 
+	case RIG_PORT_NETWORK:
+	strncpy(rs->rotport.pathname, "localhost:4533", FILPATHLEN);
+	break;
+
 	default:
 	strncpy(rs->rotport.pathname, "", FILPATHLEN);
 	}
@@ -313,12 +322,22 @@ int HAMLIB_API rot_open(ROT *rot)
 		rs->rotport.fd = status;
 		break;
 
+	case RIG_PORT_USB:
+		status = usb_port_open(&rs->rotport);
+		if (status < 0)
+			return status;
+		break;
+
 	case RIG_PORT_NONE:
 	case RIG_PORT_RPC:
 			break;	/* ez :) */
 
-	case RIG_PORT_NETWORK:	/* not implemented yet! */
-		return -RIG_ENIMPL;
+	case RIG_PORT_NETWORK:
+		status = network_open(&rs->rotport, 4533);
+		if (status < 0)
+			return status;
+		break;
+
 	default:
 		return -RIG_EINVAL;
 	}
@@ -388,6 +407,9 @@ int HAMLIB_API rot_close(ROT *rot)
 		case RIG_PORT_PARALLEL:
 			par_close(&rs->rotport);
 			break;
+		case RIG_PORT_USB:
+			usb_port_close(&rs->rotport);
+			break;
 		default:
 			close(rs->rotport.fd);
 		}
