@@ -2,7 +2,7 @@
  *  Hamlib TenTenc backend - TT-565 description
  *  Copyright (c) 2004-2008 by Stephane Fillod & Martin Ewing
  *
- *	$Id: orion.c,v 1.24 2008/02/15 23:04:51 aa6e Exp $
+ *	$Id: orion.c,v 1.26 2008/06/19 04:26:52 aa6e Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -113,11 +113,10 @@ int tt565_transaction(RIG *rig, const char *cmd, int cmd_len, char *data, int *d
         int retval, data_len_init, itry;
         struct rig_state *rs;
         static int passcount=0;
-        char cmds[64]; /* debug */
 #ifdef TT565_TIME
  	double ft1, ft2;	
 #endif
-    passcount++;        // for debugging
+	passcount++;        // for debugging
 	/* Capture buffer length for possible read re-try. */
 	data_len_init = (data && data_len) ? *data_len : 0;
 	/* Allow transaction re-tries according to capabilities. */
@@ -130,52 +129,54 @@ int tt565_transaction(RIG *rig, const char *cmd, int cmd_len, char *data, int *d
         /* no data expected, TODO: flush input? */    
         if (!data || !data_len) {
                 if (*cmd != '*') {     // i.e. was not a 'write' to rig...
-                    rig_debug(RIG_DEBUG_ERR,"reject 1\n");
-                    return -RIG_ERJCTED;    // bad news!
+                    rig_debug(RIG_DEBUG_ERR,"cmd reject 1\n");
+                    return -RIG_ERJCTED;
                     }
-                return RIG_OK;	/* normal exit if no read */
+                return RIG_OK;	/* normal exit if write, but no read */
                 }
 #ifdef TT565_TIME
 	ft1 = tt565_timenow();
 #endif
 	*data_len = data_len_init;	/* restore orig. buffer length */
-    *data_len = read_string(&rs->rigport, data, *data_len, 
+	*data_len = read_string(&rs->rigport, data, *data_len, 
 		        EOM, strlen(EOM));
-    if (!strncmp(data,"Z!",2)) {     // command unrecognized??
-        rig_debug(RIG_DEBUG_ERR,"reject 2\n");
-        return -RIG_ERJCTED;        // what is a better error return?
-        }
-    if (cmd[0] != '?') {              // was this a write to the rig?
-        rig_debug(RIG_DEBUG_ERR,"reject 3\n");
-        return -RIG_ERJCTED;        // terrible, can't get here??
-        }
-    else {                          // no, it was a 'read', phew!
-        if (!strncmp(cmd,"?V",2)) {
-            return RIG_OK;          // special case for ?V command
-            }
-        if (!strncmp(data+1,cmd+1,cmd_len-2)) {  //reponse matches cmd?
-            return RIG_OK;          // all is well, normal exit
-            }
-        else {
-            /* The command read back does not match the command that
-                was written.  What to do?  We report the problem if debugging,
-                and issue another read in hopes that
-                will make it sync up right.
-                */
-            rig_debug(RIG_DEBUG_ERR,
-                "** retry after delay (io=%d, retry=%d) **\n", 
-                passcount, itry);
-	        *data_len = data_len_init;	/* restore orig. buffer length */
-            read_string(&rs->rigport, data, *data_len, 
-		        EOM, strlen(EOM));      // purge the input stream...
-            continue;                   // now go retry the full command
-            }
-        }
+	if (!strncmp(data,"Z!",2)) {     // command unrecognized??
+        	rig_debug(RIG_DEBUG_ERR,"cmd reject 2\n");
+        	return -RIG_ERJCTED;	// what is a better error return?
+        	}
+	/* XX and ?V are oddball commands.  Thanks, Ten-Tec! */
+        if (!strncmp(cmd,"XX",2)) {	// Was it a firmware reset cmd?
+        	return RIG_OK;		// Then we accept the response.
+        	}
+        if (!strncmp(cmd,"?V",2)) {	// Was it a read firmware version cmd?
+        	return RIG_OK;		// ditto
+        	}
+	if (cmd[0] != '?') {		// was this a read cmd?
+		rig_debug(RIG_DEBUG_ERR,"cmd reject 3\n");
+		return -RIG_ERJCTED;	// No, but it should have been!
+		}
+	else {                          // Yes, it was a 'read', phew!
+	        if (!strncmp(data+1,cmd+1,cmd_len-2)) {  //reponse matches cmd?
+	            return RIG_OK;	// all is well, normal exit
+	            }
+	        else { /* The command read back does not match the command that
+	                was written.  We report the problem if debugging,
+	                and issue another read in hopes of eventual success.
+	                */
+	            rig_debug(RIG_DEBUG_ERR,
+	                "** retry after delay (io=%d, retry=%d) **\n", 
+	                passcount, itry);
+		        *data_len = data_len_init;	/* restore orig. buffer length */
+	            read_string(&rs->rigport, data, *data_len, 
+			        EOM, strlen(EOM));      // purge the input stream...
+	            continue;                   // now go retry the full command
+	            }
+	        }
 #ifdef TT565_TIME
-	    ft2 = tt565_timenow();
+	ft2 = tt565_timenow();
         if (*data_len == -RIG_ETIMEOUT)
-	    rig_debug(RIG_DEBUG_ERR,"Timeout %d: Elapsed = %f secs.\n", 
-	        itry, ft2-ft1);
+		rig_debug(RIG_DEBUG_ERR,"Timeout %d: Elapsed = %f secs.\n", 
+			itry, ft2-ft1);
 	    else
 	        rig_debug(RIG_DEBUG_ERR,
 			"Other Error #%d, itry=%d: Elapsed = %f secs.\n", 
@@ -183,7 +184,7 @@ int tt565_transaction(RIG *rig, const char *cmd, int cmd_len, char *data, int *d
 #endif 
 	}   /* end of itry loop */
 	    rig_debug(RIG_DEBUG_ERR,"** Ran out of retries io=%d **\n",
-	                             passcount);
+		passcount);
         return -RIG_ETIMEOUT;
 }
 
@@ -319,29 +320,29 @@ int tt565_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
 {
 	int cmd_len, retval;
 	char cmdbuf[TT565_BUFSIZE];
-    unsigned int myfreq;
-
-/*  ASCII mode (former)
- *	cmd_len = sprintf (cmdbuf, "*%cF%"PRIll EOM, 
- *			which_vfo(rig, vfo),
- *			(long long)freq);
- */
-
+#ifdef TT565_ASCII_FREQ
+/*  Use ASCII mode to set frequencies */
+	cmd_len = sprintf (cmdbuf, "*%cF%"PRIll EOM, 
+			which_vfo(rig, vfo),
+			(long long)freq);
+#else
+    /* Use binary mode */
     /* Set frequency using Orion's binary mode (short) sequence. 
     The short sequence transfers faster and may require less Orion
     firmware effort. */
 
     /* Construct command packet by brute force. */
-    myfreq = freq;
+	unsigned int myfreq;
+	myfreq = freq;
 	cmd_len = 7;
-    cmdbuf[0] = '*';
-    cmdbuf[1] = which_vfo(rig,vfo);
-    cmdbuf[2] = (myfreq & 0xff000000) >> 24;
-    cmdbuf[3] = (myfreq & 0x00ff0000) >> 16;
-    cmdbuf[4] = (myfreq & 0x0000ff00) >> 8;
-    cmdbuf[5] =  myfreq & 0x000000ff;
-    cmdbuf[6] = '\r';   /* i.e. EOM */
-
+	cmdbuf[0] = '*';
+	cmdbuf[1] = which_vfo(rig,vfo);
+	cmdbuf[2] = (myfreq & 0xff000000) >> 24;
+	cmdbuf[3] = (myfreq & 0x00ff0000) >> 16;
+	cmdbuf[4] = (myfreq & 0x0000ff00) >> 8;
+	cmdbuf[5] =  myfreq & 0x000000ff;
+	cmdbuf[6] = '\r';   /* i.e. EOM */
+#endif
 	retval = tt565_transaction (rig, cmdbuf, cmd_len, NULL, NULL);
 
 	return retval;
@@ -358,23 +359,27 @@ int tt565_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
 int tt565_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 {
 	int cmd_len, resp_len, retval;
-    unsigned int binf;
 	char cmdbuf[TT565_BUFSIZE], respbuf[TT565_BUFSIZE];
-
-/*  ASCII mode (former)
- *	cmd_len = sprintf(cmdbuf, "?%cF" EOM,
- *				which_vfo(rig, vfo));
- */
-
-    /* Get freq with Orion binary mode short sequence, as above. */
+	unsigned int binf;
+#ifdef TT565_ASCII_FREQ
+	/*  use ASCII mode */
+	cmd_len = sprintf(cmdbuf, "?%cF" EOM,
+				which_vfo(rig, vfo));
+#else
+	/* Get freq with Orion binary mode short sequence. */
 	cmd_len = sprintf(cmdbuf, "?%c" EOM,
 				which_vfo(rig, vfo));
-
+#endif
 	resp_len = sizeof(respbuf);	
 	retval = tt565_transaction (rig, cmdbuf, cmd_len, respbuf, &resp_len);
 
 	if (retval != RIG_OK)
 		return retval;
+#ifdef TT565_ASCII_FREQ
+	respbuf[12] = '\0';
+	sscanf(respbuf+3,"%8u",&binf);
+	*freq = (freq_t) binf;
+#else
     /* Test for valid binary mode return. */
 	if (respbuf[1] != which_vfo(rig,vfo) || resp_len <= 5) {
 		rig_debug(RIG_DEBUG_ERR, "%s: unexpected answer '%s'\n",
@@ -382,9 +387,10 @@ int tt565_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 		return -RIG_EPROTO;
 	}
     /* Convert binary to integer, endedness independent */
-    binf = (unsigned char)respbuf[2]<<24 | (unsigned char)respbuf[3]<<16
-         | (unsigned char)respbuf[4]<<8  | (unsigned char)respbuf[5];
+	binf = (unsigned char)respbuf[2]<<24 | (unsigned char)respbuf[3]<<16
+		| (unsigned char)respbuf[4]<<8  | (unsigned char)respbuf[5];
 	*freq = (freq_t) binf;
+#endif
 	return RIG_OK;
 }
 
