@@ -5,7 +5,7 @@
  * It takes commands in interactive mode as well as 
  * from command line options.
  *
- * $Id: rigctl.c,v 1.66 2008/01/12 00:36:58 n0nb Exp $  
+ * $Id: rigctl.c,v 1.68 2008/10/27 22:23:36 fillods Exp $  
  *
  *
  * This program is free software; you can redistribute it and/or
@@ -84,6 +84,7 @@ static struct option long_options[] =
 
 int interactive = 1;    /* if no cmd on command line, switch to interactive */
 int prompt = 1;         /* Print prompt in rigctl */
+int opt_end= 0;         /* only used by rigctld */
 
 int main (int argc, char *argv[])
 {
@@ -281,11 +282,11 @@ int main (int argc, char *argv[])
 	 * print out conf parameters
 	 */
 	if (show_conf) {
-		rig_token_foreach(my_rig, print_conf_list, (rig_ptr_t)my_rig);
+		dumpconf(my_rig, stdout);
 	}
 
 	/*
-	 * print out conf parameters, and exists immediately
+	 * print out capabilities, and exists immediately
 	 * We may be interested only in only caps, and rig_open may fail.
 	 */
 	if (dump_caps_opt) {
