@@ -5,7 +5,7 @@
  * It takes commands in interactive mode as well as 
  * from command line options.
  *
- * $Id: rigctl_parse.c,v 1.4 2008/01/12 00:36:58 n0nb Exp $  
+ * $Id: rigctl_parse.c,v 1.9 2008/10/27 22:23:36 fillods Exp $  
  *
  *
  * This program is free software; you can redistribute it and/or
@@ -98,6 +98,7 @@ declare_proto_rig(get_vfo);
 declare_proto_rig(set_ptt);
 declare_proto_rig(get_ptt);
 declare_proto_rig(get_ptt);
+declare_proto_rig(get_dcd);
 declare_proto_rig(set_rptr_shift);
 declare_proto_rig(get_rptr_shift);
 declare_proto_rig(set_rptr_offs);
@@ -106,6 +107,10 @@ declare_proto_rig(set_ctcss_tone);
 declare_proto_rig(get_ctcss_tone);
 declare_proto_rig(set_dcs_code);
 declare_proto_rig(get_dcs_code);
+declare_proto_rig(set_ctcss_sql);
+declare_proto_rig(get_ctcss_sql);
+declare_proto_rig(set_dcs_sql);
+declare_proto_rig(get_dcs_sql);
 declare_proto_rig(set_split_freq);
 declare_proto_rig(get_split_freq);
 declare_proto_rig(set_split_mode);
@@ -132,6 +137,8 @@ declare_proto_rig(set_trn);
 declare_proto_rig(get_trn);
 declare_proto_rig(get_info);
 declare_proto_rig(dump_caps);
+declare_proto_rig(dump_conf);
+declare_proto_rig(dump_state);
 declare_proto_rig(set_ant);
 declare_proto_rig(get_ant);
 declare_proto_rig(reset);
@@ -139,7 +146,8 @@ declare_proto_rig(send_morse);
 declare_proto_rig(send_cmd);
 declare_proto_rig(set_powerstat);
 declare_proto_rig(get_powerstat);
-
+declare_proto_rig(send_dtmf);
+declare_proto_rig(recv_dtmf);
 
 
 /*
@@ -182,6 +190,10 @@ static struct test_table test_list[] = {
 	{ 'c', "get_ctcss_tone", get_ctcss_tone, ARG_OUT, "CTCSS tone" },
 	{ 'D', "set_dcs_code", set_dcs_code, ARG_IN, "DCS code" },
 	{ 'd', "get_dcs_code", get_dcs_code, ARG_OUT, "DCS code" },
+	{ 0x90, "set_ctcss_sql", set_ctcss_sql, ARG_IN, "CTCSS tone" },
+	{ 0x91, "get_ctcss_sql", get_ctcss_sql, ARG_OUT, "CTCSS tone" },
+	{ 0x92, "set_dcs_sql", set_dcs_sql, ARG_IN, "DCS code" },
+	{ 0x93, "get_dcs_sql", get_dcs_sql, ARG_OUT, "DCS code" },
 	{ 'V', "set_vfo", set_vfo, ARG_IN|ARG_NOVFO, "VFO" },
 	{ 'v', "get_vfo", get_vfo, ARG_OUT, "VFO" },
 	{ 'T', "set_ptt", set_ptt, ARG_IN, "PTT" },
@@ -200,11 +212,16 @@ static struct test_table test_list[] = {
 	{ 'y', "get_ant", get_ant, ARG_OUT, "Antenna" },
 	{ 0x87, "set_powerstat", set_powerstat, ARG_IN|ARG_NOVFO, "Status" },
 	{ 0x88, "get_powerstat", get_powerstat, ARG_OUT|ARG_NOVFO, "Status" },
+	{ 0x89, "send_dtmf", send_dtmf, ARG_IN, "Digits" },
+	{ 0x8a, "recv_dtmf", recv_dtmf, ARG_OUT, "Digits" },
 	{ '*', "reset", reset, ARG_IN, "Reset" },
 	{ '2', "power2mW", power2mW, ARG_NOVFO },
 	{ 'w', "send_cmd", send_cmd, ARG_IN1|ARG_IN_LINE|ARG_OUT2|ARG_NOVFO, "Cmd", "Reply" },
-	{ '1', "dump_caps", dump_caps, ARG_NOVFO },
 	{ 'b', "send_morse", send_morse, ARG_IN|ARG_IN_LINE, "Morse" },
+	{ 0x8b, "get_dcd", get_dcd, ARG_OUT, "DCD" },
+	{ '1', "dump_caps", dump_caps, ARG_NOVFO },
+	{ '3', "dump_conf", dump_conf, ARG_NOVFO },
+	{ 0x8f,"dump_state", dump_state, ARG_OUT|ARG_NOVFO },
 	/* next one is 0x89 */
 	{ 0x00, "", NULL },
 
@@ -257,6 +274,7 @@ static int scanfc(FILE *fin, const char *format, void *p)
 
 extern int interactive;
 extern int prompt;
+extern int opt_end;
 
 
 int rigctl_parse(RIG *my_rig, FILE *fin, FILE *fout, char *argv[], int argc)
@@ -322,6 +340,7 @@ int rigctl_parse(RIG *my_rig, FILE *fin, FILE *fout, char *argv[], int argc)
 				return 1;
 		if (cmd == '?') {
 			usage_rig(fout);
+			fflush(fout);
 			return 0;
 		}
 	} else {
@@ -396,7 +415,7 @@ int rigctl_parse(RIG *my_rig, FILE *fin, FILE *fout, char *argv[], int argc)
 			p1 = argv[optind++];
 		}
 	}
-	if ((cmd_entry->flags & ARG_IN2) && cmd_entry->arg2) {
+	if (p1 && p1[0]!='?' && (cmd_entry->flags & ARG_IN2) && cmd_entry->arg2) {
 		if (interactive) {
 			if (prompt)
 				fprintf(fout, "%s: ", cmd_entry->arg2);
@@ -412,7 +431,7 @@ int rigctl_parse(RIG *my_rig, FILE *fin, FILE *fout, char *argv[], int argc)
 			p2 = argv[optind++];
 		}
 	}
-	if ((cmd_entry->flags & ARG_IN3) && cmd_entry->arg3) {
+	if (p1 && p1[0]!='?' && (cmd_entry->flags & ARG_IN3) && cmd_entry->arg3) {
 		if (interactive) {
 			if (prompt)
 				fprintf(fout, "%s: ", cmd_entry->arg3);
@@ -448,12 +467,22 @@ int rigctl_parse(RIG *my_rig, FILE *fin, FILE *fout, char *argv[], int argc)
 	pthread_mutex_unlock(&rig_mutex);
 #endif
 
-	fflush(fout);
-		
 	if (retcode != RIG_OK) {
-  		fprintf(fout, "%s: error = %s\n", cmd_entry->name, rigerror(retcode));
+		if (interactive && !prompt)
+  			fprintf(fout, NETRIGCTL_RET "%d\n", retcode);	/* only for rigctld */
+		else
+  			fprintf(fout, "%s: error = %s\n", cmd_entry->name, rigerror(retcode));
+	} else {
+		if (interactive && !prompt) {	/* only for rigctld */
+			if (!(cmd_entry->flags & ARG_OUT) && !opt_end) /* netrigctl RIG_OK */
+				fprintf(fout, NETRIGCTL_RET "0\n");
+			else if ((cmd_entry->flags & ARG_OUT) && opt_end) /* Nate's protocol */
+				fprintf(fout, "END\n");
+		}
 	}
 
+	fflush(fout);
+
 	return 0;
 }
 
@@ -466,23 +495,25 @@ void version()
 
 void usage_rig(FILE *fout)
 {
-	int i;
+	int i, nbspaces;
 
-	fprintf(fout, "Commands (may not be available for this rig):\n");
+	fprintf(fout, "Commands (some may not be available for this rig):\n");
 	for (i=0; test_list[i].cmd != 0; i++) {
 		fprintf(fout, "%c: %-16s(", isprint(test_list[i].cmd) ? 
 				test_list[i].cmd:'?', test_list[i].name);
 
+		nbspaces = 18;
 		if (test_list[i].arg1 && (test_list[i].flags&ARG_IN1))
-				fprintf(fout, "%s", test_list[i].arg1);
+				nbspaces -= fprintf(fout, "%s", test_list[i].arg1);
 		if (test_list[i].arg2 && (test_list[i].flags&ARG_IN2))
-				fprintf(fout, ",%s", test_list[i].arg2);
+				nbspaces -= fprintf(fout, ",%s", test_list[i].arg2);
 		if (test_list[i].arg3 && (test_list[i].flags&ARG_IN3))
-				fprintf(fout, ",%s", test_list[i].arg3);
-		fprintf(fout, ")  \t");
+				nbspaces -= fprintf(fout, ",%s", test_list[i].arg3);
 
 		if (i%2)
-			fprintf(fout, "\n");
+			fprintf(fout, ")\n");
+		else
+			fprintf(fout, ")%*s", nbspaces, " ");
 	}
 }
 
@@ -595,10 +626,7 @@ declare_proto_rig(get_freq)
 
 	if (interactive && prompt)
 		fprintf(fout, "%s: ", cmd->arg1); /* i.e. "Frequency" */
-    fprintf(fout, "%"PRIll"\n", (long long)freq);
-
-	if (interactive && !prompt)             /* only for rigctld */
-		fprintf(fout, "END\n");
+	fprintf(fout, "%"PRIll"\n", (long long)freq);
 
 	return status;
 }
@@ -625,9 +653,6 @@ declare_proto_rig(get_rit)
 		fprintf(fout, "%s: ", cmd->arg1);
 	fprintf(fout, "%ld\n", rit);
 
-	if (interactive && !prompt)             /* only for rigctld */
-		fprintf(fout, "END\n");
-
 	return status;
 }
 
@@ -653,9 +678,6 @@ declare_proto_rig(get_xit)
 		fprintf(fout, "%s: ", cmd->arg1);
 	fprintf(fout, "%ld\n", xit);
 
-	if (interactive && !prompt)             /* only for rigctld */
-		fprintf(fout, "END\n");
-
 	return status;
 }
 
@@ -665,6 +687,14 @@ declare_proto_rig(set_mode)
 	rmode_t mode;
 	pbwidth_t width;
 
+#if 0
+	if (!strcmp(arg1, "?")) {
+		char s[SPRINTF_MAX_SIZE];
+		sprintf_mode(s, rig->state.modes);
+		fprintf(fout, "%s\n", s);
+		return RIG_OK;
+	}
+#endif
 	mode = rig_parse_mode(arg1);
 	sscanf(arg2, "%ld", &width);
 	return rig_set_mode(rig, vfo, mode, width);
@@ -687,9 +717,6 @@ declare_proto_rig(get_mode)
 		fprintf(fout, "%s: ", cmd->arg2);
 	fprintf(fout, "%ld\n", width);
 
-	if (interactive && !prompt)             /* only for rigctld */
-		fprintf(fout, "END\n");
-
 	return status;
 }
 
@@ -711,9 +738,6 @@ declare_proto_rig(get_vfo)
 		fprintf(fout, "%s: ", cmd->arg1);
 	fprintf(fout, "%s\n", rig_strvfo(vfo));
 
-	if (interactive && !prompt)             /* only for rigctld */
-		fprintf(fout, "END\n");
-
 	return status;
 }
 
@@ -739,8 +763,20 @@ declare_proto_rig(get_ptt)
 		fprintf(fout, "%s: ", cmd->arg1);
 	fprintf(fout, "%d\n", ptt);
 
-	if (interactive && !prompt)             /* only for rigctld */
-		fprintf(fout, "END\n");
+	return status;
+}
+
+declare_proto_rig(get_dcd)
+{
+	int status;
+	dcd_t dcd;
+
+	status = rig_get_dcd(rig, vfo, &dcd);
+	if (status != RIG_OK)
+		return status;
+	if (interactive && prompt)
+		fprintf(fout, "%s: ", cmd->arg1);
+	fprintf(fout, "%d\n", dcd);
 
 	return status;
 }
@@ -767,9 +803,6 @@ declare_proto_rig(get_rptr_shift)
 		fprintf(fout, "%s: ", cmd->arg1);
 	fprintf(fout, "%s\n", rig_strptrshift(rptr_shift));
 
-	if (interactive && !prompt)             /* only for rigctld */
-		fprintf(fout, "END\n");
-
 	return status;
 }
 
@@ -795,9 +828,6 @@ declare_proto_rig(get_rptr_offs)
 		fprintf(fout, "%s: ", cmd->arg1);
 	fprintf(fout, "%ld\n", rptr_offs);
 
-	if (interactive && !prompt)             /* only for rigctld */
-		fprintf(fout, "END\n");
-
 	return status;
 }
 
@@ -823,9 +853,6 @@ declare_proto_rig(get_ctcss_tone)
 		fprintf(fout, "%s: ", cmd->arg1);
 	fprintf(fout, "%d\n", tone);
 
-	if (interactive && !prompt)             /* only for rigctld */
-		fprintf(fout, "END\n");
-
 	return status;
 }
 
@@ -851,8 +878,51 @@ declare_proto_rig(get_dcs_code)
 		fprintf(fout, "%s: ", cmd->arg1);
 	fprintf(fout, "%d\n", code);
 
-	if (interactive && !prompt)             /* only for rigctld */
-		fprintf(fout, "END\n");
+	return status;
+}
+
+declare_proto_rig(set_ctcss_sql)
+{
+	tone_t tone;
+
+	sscanf(arg1, "%d", &tone);
+	return rig_set_ctcss_sql(rig, vfo, tone);
+}
+
+declare_proto_rig(get_ctcss_sql)
+{
+	int status;
+	tone_t tone;
+
+	status = rig_get_ctcss_sql(rig, vfo, &tone);
+	if (status != RIG_OK)
+		return status;
+	if (interactive && prompt)
+		fprintf(fout, "%s: ", cmd->arg1);
+	fprintf(fout, "%d\n", tone);
+
+	return status;
+}
+
+declare_proto_rig(set_dcs_sql)
+{
+	tone_t code;
+
+	sscanf(arg1, "%d", &code);
+	return rig_set_dcs_sql(rig, vfo, code);
+}
+
+declare_proto_rig(get_dcs_sql)
+{
+	int status;
+	tone_t code;
+
+	status = rig_get_dcs_sql(rig, vfo, &code);
+	if (status != RIG_OK)
+		return status;
+	if (interactive && prompt)
+		fprintf(fout, "%s: ", cmd->arg1);
+	fprintf(fout, "%d\n", code);
 
 	return status;
 }
@@ -879,9 +949,6 @@ declare_proto_rig(get_split_freq)
 		fprintf(fout, "%s: ", cmd->arg1);
 	fprintf(fout, "%"PRIll"\n", (long long)txfreq);
 
-	if (interactive && !prompt)             /* only for rigctld */
-		fprintf(fout, "END\n");
-
 	return status;
 }
 
@@ -891,6 +958,14 @@ declare_proto_rig(set_split_mode)
 	rmode_t mode;
 	int     width;
 
+#if 0
+	if (!strcmp(arg1, "?")) {
+		char s[SPRINTF_MAX_SIZE];
+		sprintf_mode(s, rig->state.modes);
+		fprintf(fout, "%s\n", s);
+		return RIG_OK;
+	}
+#endif
 	mode = rig_parse_mode(arg1);
 	sscanf(arg2, "%d", &width);
 	return rig_set_split_mode(rig, vfo, mode, (pbwidth_t) width);
@@ -913,9 +988,6 @@ declare_proto_rig(get_split_mode)
 		fprintf(fout, "%s: ", cmd->arg2);
 	fprintf(fout, "%ld\n", width);
 
-	if (interactive && !prompt)             /* only for rigctld */
-		fprintf(fout, "END\n");
-
 	return status;
 }
 
@@ -945,9 +1017,6 @@ declare_proto_rig(get_split_vfo)
 		fprintf(fout, "%s: ", cmd->arg2);
 	fprintf(fout, "%s\n", rig_strvfo(tx_vfo));
 
-	if (interactive && !prompt)             /* only for rigctld */
-		fprintf(fout, "END\n");
-
 	return status;
 }
 
@@ -973,9 +1042,6 @@ declare_proto_rig(get_ts)
 		fprintf(fout, "%s: ", cmd->arg1);
 	fprintf(fout, "%ld\n", ts);
 
-	if (interactive && !prompt)             /* only for rigctld */
-		fprintf(fout, "END\n");
-
 	return status;
 }
 
@@ -998,9 +1064,6 @@ declare_proto_rig(power2mW)
 	status = rig_power2mW(rig, &mwp, power, freq, (rmode_t) mode);
 	fprintf(fout, "Power: %d mW\n", mwp);
 
-    if (interactive && !prompt)             /* only for rigctld */
-		fprintf(fout, "END\n");
-
 	return status;
 }
 
@@ -1018,6 +1081,13 @@ declare_proto_rig(set_level)
 	setting_t level;
 	value_t val;
 
+	if (!strcmp(arg1, "?")) {
+		char s[SPRINTF_MAX_SIZE];
+		sprintf_level(s, rig->state.has_set_level);
+		fprintf(fout, "%s\n", s);
+		return RIG_OK;
+	}
+
 	level = rig_parse_level(arg1);
 	if (!rig_has_set_level(rig, level)) {
 		const struct confparams *cfp;
@@ -1058,6 +1128,13 @@ declare_proto_rig(get_level)
 	setting_t level;
 	value_t val;
 
+	if (!strcmp(arg1, "?")) {
+		char s[SPRINTF_MAX_SIZE];
+		sprintf_level(s, rig->state.has_get_level);
+		fprintf(fout, "%s\n", s);
+		return RIG_OK;
+	}
+
 	level = rig_parse_level(arg1);
 	if (!rig_has_get_level(rig, level)) {
 		const struct confparams *cfp;
@@ -1100,9 +1177,6 @@ declare_proto_rig(get_level)
 	else
 		fprintf(fout, "%d\n", val.i);
 
-	if (interactive && !prompt)             /* only for rigctld */
-		fprintf(fout, "END\n");
-
 	return status;
 }
 
@@ -1112,8 +1186,15 @@ declare_proto_rig(set_func)
 	setting_t func;
 	int func_stat;
 
+	if (!strcmp(arg1, "?")) {
+		char s[SPRINTF_MAX_SIZE];
+		sprintf_func(s, rig->state.has_set_func);
+		fprintf(fout, "%s\n", s);
+		return RIG_OK;
+	}
+
 	func = rig_parse_func(arg1);
-	sscanf(arg2, "%d", (int*)&func_stat);
+	sscanf(arg2, "%d", &func_stat);
 	return rig_set_func(rig, vfo, func, func_stat);
 }
 
@@ -1124,6 +1205,13 @@ declare_proto_rig(get_func)
 	setting_t func;
 	int func_stat;
 
+	if (!strcmp(arg1, "?")) {
+		char s[SPRINTF_MAX_SIZE];
+		sprintf_func(s, rig->state.has_get_func);
+		fprintf(fout, "%s\n", s);
+		return RIG_OK;
+	}
+
 	func = rig_parse_func(arg1);
 	status = rig_get_func(rig, vfo, func, &func_stat);
 	if (status != RIG_OK)
@@ -1132,9 +1220,6 @@ declare_proto_rig(get_func)
 		fprintf(fout, "%s: ", cmd->arg2);
 	fprintf(fout, "%d\n", func_stat);
 
-	if (interactive && !prompt)             /* only for rigctld */
-		fprintf(fout, "END\n");
-
 	return status;
 }
 
@@ -1144,6 +1229,13 @@ declare_proto_rig(set_parm)
 	setting_t parm;
 	value_t val;
 
+	if (!strcmp(arg1, "?")) {
+		char s[SPRINTF_MAX_SIZE];
+		sprintf_parm(s, rig->state.has_set_parm);
+		fprintf(fout, "%s\n", s);
+		return RIG_OK;
+	}
+
 	parm = rig_parse_parm(arg1);
 
 	if (!rig_has_set_parm(rig, parm)) {
@@ -1185,6 +1277,13 @@ declare_proto_rig(get_parm)
 	setting_t parm;
 	value_t val;
 
+	if (!strcmp(arg1, "?")) {
+		char s[SPRINTF_MAX_SIZE];
+		sprintf_parm(s, rig->state.has_get_parm);
+		fprintf(fout, "%s\n", s);
+		return RIG_OK;
+	}
+
 	parm = rig_parse_parm(arg1);
 	if (!rig_has_get_parm(rig, parm)) {
 		const struct confparams *cfp;
@@ -1227,9 +1326,6 @@ declare_proto_rig(get_parm)
 	else
 		fprintf(fout, "%d\n", val.i);
 
-	if (interactive && !prompt)             /* only for rigctld */
-		fprintf(fout, "END\n");
-
 	return status;
 }
 
@@ -1264,9 +1360,6 @@ declare_proto_rig(get_mem)
 		fprintf(fout, "%s: ", cmd->arg1);
 	fprintf(fout, "%d\n", ch);
 
-    if (interactive && !prompt)             /* only for rigctld */
-		fprintf(fout, "END\n");
-
 	return status;
 }
 
@@ -1275,6 +1368,13 @@ declare_proto_rig(vfo_op)
 {
 	vfo_op_t op;
 
+	if (!strcmp(arg1, "?")) {
+		char s[SPRINTF_MAX_SIZE];
+		sprintf_vfop(s, rig->caps->vfo_ops);
+		fprintf(fout, "%s\n", s);
+		return RIG_OK;
+	}
+
 	op = rig_parse_vfo_op(arg1);
 	return rig_vfo_op(rig, vfo, op);
 }
@@ -1285,6 +1385,13 @@ declare_proto_rig(scan)
 	scan_t op;
 	int ch;
 
+	if (!strcmp(arg1, "?")) {
+		char s[SPRINTF_MAX_SIZE];
+		sprintf_scan(s, rig->caps->scan_ops);
+		fprintf(fout, "%s\n", s);
+		return RIG_OK;
+	}
+
 	op = rig_parse_scan(arg1);
 	sscanf(arg2, "%d", &ch);
 	return rig_scan(rig, vfo, op, ch);
@@ -1293,9 +1400,8 @@ declare_proto_rig(scan)
 /* 'H' */
 declare_proto_rig(set_channel)
 {
-	channel_cap_t *mem_caps = NULL;
-	chan_t *chan_list;
-	int i;
+	const channel_cap_t *mem_caps = NULL;
+	const chan_t *chan_list;
 	channel_t chan;
 
 	return -RIG_ENIMPL;
@@ -1307,14 +1413,9 @@ declare_proto_rig(set_channel)
 		/*
 		 * find mem_caps in caps, we'll need it later
 		 */
-		chan_list = rig->caps->chan_list;
-		for (i=0; i<CHANLSTSIZ && !RIG_IS_CHAN_END(chan_list[i]); i++) {
-			if (chan.channel_num >= chan_list[i].start &&
-					chan.channel_num <= chan_list[i].end) {
-				mem_caps = &chan_list[i].mem_caps;
-				break;
-			}
-		}
+		chan_list = rig_lookup_mem_caps(rig, chan.channel_num);
+		if (chan_list)
+			mem_caps = &chan_list->mem_caps;
 
 	} else {
 		chan.vfo = rig_parse_vfo(arg1);
@@ -1413,9 +1514,6 @@ declare_proto_rig(get_channel)
 		return status;
 	dump_chan(fout, rig, &chan);
 
-	if (interactive && !prompt)             /* only for rigctld */
-		fprintf(fout, "END\n");
-
 	return status;
 }
 
@@ -1488,9 +1586,6 @@ declare_proto_rig(get_trn)
 		fprintf(fout, "%s: ", cmd->arg1);
 	fprintf(fout, "%d\n", trn);
 
-	if (interactive && !prompt)             /* only for rigctld */
-		fprintf(fout, "END\n");
-
 	return status;
 }
 
@@ -1504,9 +1599,6 @@ declare_proto_rig(get_info)
 		fprintf(fout, "%s: ", cmd->arg1);
 	fprintf(fout, "%s\n", s ? s : "None");
 
-	if (interactive && !prompt)             /* only for rigctld */
-		fprintf(fout, "END\n");
-
 	return RIG_OK;
 }
 
@@ -1558,8 +1650,9 @@ void dump_chan(FILE *fout, RIG *rig, channel_t *chan)
 		setting_t level = rig_idx2setting(idx);
 		const char *level_s;
 
-		if (!rig_has_set_level(rig, level))
-				continue;
+		if (!RIG_LEVEL_SET(level) ||
+				(!rig_has_set_level(rig, level) && !rig_has_get_level(rig, level)))
+			continue;
 		level_s = rig_strlevel(level);
 		if (!level_s)
 				continue;	/* duh! */
@@ -1580,8 +1673,94 @@ declare_proto_rig(dump_caps)
 {
 	dumpcaps(rig, fout);
 
-	if (interactive && !prompt)             /* only for rigctld */
-		fprintf(fout, "END\n");
+	return RIG_OK;
+}
+
+/* For rigctld internal use */
+declare_proto_rig(dump_state)
+{
+	int i;
+	struct rig_state *rs = &rig->state;
+
+	/*
+	 * - Protocol version
+	 */
+#define RIGCTLD_PROT_VER 0
+	fprintf(fout, "%d\n", RIGCTLD_PROT_VER);
+	fprintf(fout, "%d\n", rig->caps->rig_model);
+	fprintf(fout, "%d\n", rs->itu_region);
+
+	for (i=0; i<FRQRANGESIZ && !RIG_IS_FRNG_END(rs->rx_range_list[i]); i++)
+		fprintf(fout, "%"FREQFMT" %"FREQFMT" 0x%x %d %d 0x%x 0x%x\n",
+				rs->rx_range_list[i].start,
+				rs->rx_range_list[i].end,
+				rs->rx_range_list[i].modes,
+				rs->rx_range_list[i].low_power,
+				rs->rx_range_list[i].high_power,
+				rs->rx_range_list[i].vfo,
+				rs->rx_range_list[i].ant
+				);
+	fprintf(fout, "0 0 0 0 0 0 0\n");
+
+	for (i=0; i<FRQRANGESIZ && !RIG_IS_FRNG_END(rs->tx_range_list[i]); i++)
+		fprintf(fout, "%"FREQFMT" %"FREQFMT" 0x%x %d %d 0x%x 0x%x\n",
+				rs->tx_range_list[i].start,
+				rs->tx_range_list[i].end,
+				rs->tx_range_list[i].modes,
+				rs->tx_range_list[i].low_power,
+				rs->tx_range_list[i].high_power,
+				rs->tx_range_list[i].vfo,
+				rs->tx_range_list[i].ant
+				);
+	fprintf(fout, "0 0 0 0 0 0 0\n");
+
+	for (i=0; i<TSLSTSIZ && !RIG_IS_TS_END(rs->tuning_steps[i]); i++)
+		fprintf(fout, "0x%x %ld\n", 
+				rs->tuning_steps[i].modes,
+				rs->tuning_steps[i].ts);
+	fprintf(fout, "0 0\n");
+
+	for (i=0; i<FLTLSTSIZ && !RIG_IS_FLT_END(rs->filters[i]); i++)
+		fprintf(fout, "0x%x %ld\n", 
+				rs->filters[i].modes,
+				rs->filters[i].width);
+	fprintf(fout, "0 0\n");
+
+#if 0
+	chan_t chan_list[CHANLSTSIZ]; /*!< Channel list, zero ended */
+#endif
+
+	fprintf(fout, "%ld\n", rs->max_rit);
+	fprintf(fout, "%ld\n", rs->max_xit);
+	fprintf(fout, "%ld\n", rs->max_ifshift);
+	fprintf(fout, "%d\n", rs->announces);
+
+	for (i=0; i<MAXDBLSTSIZ && rs->preamp[i]; i++)
+		fprintf(fout, "%d ", rs->preamp[i]);
+	fprintf(fout, "\n");
+	for (i=0; i<MAXDBLSTSIZ && rs->attenuator[i]; i++)
+		fprintf(fout, "%d ", rs->attenuator[i]);
+	fprintf(fout, "\n");
+
+	fprintf(fout, "0x%lx\n", rs->has_get_func);
+	fprintf(fout, "0x%lx\n", rs->has_set_func);
+	fprintf(fout, "0x%lx\n", rs->has_get_level);
+	fprintf(fout, "0x%lx\n", rs->has_set_level);
+	fprintf(fout, "0x%lx\n", rs->has_get_parm);
+	fprintf(fout, "0x%lx\n", rs->has_set_parm);
+
+#if 0
+	gran_t level_gran[RIG_SETTING_MAX];   /*!< level granularity */
+	gran_t parm_gran[RIG_SETTING_MAX];    /*!< parm granularity */
+#endif
+
+	return RIG_OK;
+}
+
+/* '3' */
+declare_proto_rig(dump_conf)
+{
+	dumpconf(rig, fout);
 
 	return RIG_OK;
 }
@@ -1608,9 +1787,6 @@ declare_proto_rig(get_ant)
 		fprintf(fout, "%s: ", cmd->arg1);
 	fprintf(fout, "%d\n", rig_setting2idx(ant));
 
-	if (interactive && !prompt)             /* only for rigctld */
-		fprintf(fout, "END\n");
-
 	return status;
 }
 
@@ -1629,6 +1805,28 @@ declare_proto_rig(send_morse)
 	return rig_send_morse(rig, vfo, arg1);
 }
 
+declare_proto_rig(send_dtmf)
+{
+	return rig_send_dtmf(rig, vfo, arg1);
+}
+
+declare_proto_rig(recv_dtmf)
+{
+	int status;
+	int len;
+	char digits[MAXARGSZ];
+
+	len = MAXARGSZ-1;
+	status = rig_recv_dtmf(rig, vfo, digits, &len);
+	if (status != RIG_OK)
+		return status;
+	if (interactive && prompt)
+		fprintf(fout, "%s: ", cmd->arg1);
+	fprintf(fout, "%s\n", digits);
+
+	return status;
+}
+
 /* '0x87' */
 declare_proto_rig(set_powerstat)
 {
@@ -1651,9 +1849,6 @@ declare_proto_rig(get_powerstat)
 		fprintf(fout, "%s: ", cmd->arg1);
 	fprintf(fout, "%d\n", stat);
 
-	if (interactive && !prompt)             /* only for rigctld */
-		fprintf(fout, "END\n");
-
 	return status;
 }
 
@@ -1728,9 +1923,6 @@ declare_proto_rig(send_cmd)
 	if (retval > 0 || retval == -RIG_ETIMEOUT)
 		retval = RIG_OK;
 
-	if (interactive && !prompt)             /* only for rigctld */
-		fprintf(fout, "END\n");
-
 	return retval;
 }
 
