@@ -4,7 +4,7 @@
  * This program test/control a radio using Hamlib.
  * It takes commands from network connection.
  *
- * $Id: rigctld.c,v 1.4 2008/01/12 00:36:58 n0nb Exp $  
+ * $Id: rigctld.c,v 1.9 2008/10/27 22:23:36 fillods Exp $  
  *
  *
  * This program is free software; you can redistribute it and/or
@@ -37,11 +37,22 @@
 #include <getopt.h>
 
 #include <sys/types.h>          /* See NOTES */
-#include <sys/socket.h>
-#include <netinet/in.h>
-#include <arpa/inet.h>
 
+#ifdef HAVE_NETINET_IN_H
+#include <netinet/in.h>
+#endif
+#ifdef HAVE_ARPA_INET_H
+#include <arpa/inet.h>
+#endif
+#ifdef HAVE_SYS_SOCKET_H
+#include <sys/socket.h>
+#elif HAVE_WS2TCPIP_H
+#include <ws2tcpip.h>
+#endif
+
+#ifdef HAVE_PTHREAD
 #include <pthread.h>
+#endif
 
 #include <hamlib/rig.h>
 #include "misc.h"
@@ -57,7 +68,7 @@
  * NB: do NOT use -W since it's reserved by POSIX.
  * TODO: add an option to read from a file
  */
-#define SHORT_OPTIONS "m:r:p:d:P:D:s:c:lC:t:LuovhV"
+#define SHORT_OPTIONS "m:r:p:d:P:D:s:c:lC:t:T:LeuovhV"
 static struct option long_options[] =
 {
 	{"model",    1, 0, 'm'},
@@ -69,11 +80,13 @@ static struct option long_options[] =
 	{"serial-speed", 1, 0, 's'},
 	{"civaddr",  1, 0, 'c'},
 	{"port",  1, 0, 't'},
+	{"listen-addr",  1, 0, 'T'},
 	{"list",     0, 0, 'l'},
 	{"set-conf", 1, 0, 'C'},
 	{"show-conf",0, 0, 'L'},
 	{"dump-caps",  0, 0, 'u'},
 	{"vfo",  0, 0, 'o'},
+	{"end-marker", 0, 0, 'e'},
 	{"verbose",  0, 0, 'v'},
 	{"help",     0, 0, 'h'},
 	{"version",  0, 0, 'V'},
@@ -92,8 +105,10 @@ void usage(void);
 
 int interactive = 1;    /* no cmd because of daemon */
 int prompt= 0 ;         /* Daemon mode for rigparse return string */
+int opt_end= 0 ;        /* END marker for rigctld */
 
 int portno = 4532;
+uint32_t src_addr = INADDR_ANY;
 
 #define MAXCONFLEN 128
 
@@ -118,6 +133,7 @@ int main (int argc, char *argv[])
 	int sock_listen;
 	struct sockaddr_in serv_addr;
 	int reuseaddr = 1;
+	int a0,a1,a2,a3;
 
 	while(1) {
 		int c;
@@ -231,6 +247,17 @@ int main (int argc, char *argv[])
 				}
 				portno = atoi(optarg);
 				break;
+			case 'T':
+				if (!optarg) {
+					usage();	/* wrong arg count */
+					exit(1);
+				}
+				if (4 != sscanf(optarg, "%d.%d.%d.%d", &a0,&a1,&a2,&a3)) {
+					usage();	/* wrong arg count */
+					exit(1);
+				}
+				src_addr = (a0<<24)|(a1<<16)|(a2<<8)|a3;
+				break;
 			case 'o':
 				vfo_mode++;
 				break;
@@ -246,6 +273,9 @@ int main (int argc, char *argv[])
 			case 'u':
 				dump_caps_opt++;
 				break;
+			case 'e':
+				opt_end = 1;
+				break;
 			default:
 				usage();	/* unknown option? */
 				exit(1);
@@ -326,13 +356,15 @@ int main (int argc, char *argv[])
 	 * Prepare listening socket
 	 */
 	sock_listen = socket(AF_INET, SOCK_STREAM, 0); 
-	if (sock_listen < 0) 
+	if (sock_listen < 0) {
 		perror("ERROR opening socket");
+		exit(2);
+	}
 	memset((char *) &serv_addr, 0, sizeof(serv_addr));
 
 	serv_addr.sin_family = AF_INET;
 	serv_addr.sin_port = htons(portno);
-	serv_addr.sin_addr.s_addr = INADDR_ANY;
+	serv_addr.sin_addr.s_addr = htonl(src_addr);
 
 
 	if (setsockopt(sock_listen, SOL_SOCKET, SO_REUSEADDR,
@@ -353,7 +385,10 @@ int main (int argc, char *argv[])
 	 * main loop accepting connections
 	 */
 	do {
+#ifdef HAVE_PTHREAD
 		pthread_t thread;
+		pthread_attr_t attr;
+#endif
 		struct handle_data *arg;
 
 		arg = malloc(sizeof(struct handle_data));
@@ -375,11 +410,18 @@ int main (int argc, char *argv[])
 				inet_ntoa(arg->cli_addr.sin_addr),
 				ntohs(arg->cli_addr.sin_port));
 
-		retcode = pthread_create(&thread, NULL, handle_socket, arg);
-		if (retcode < 0) {
+#ifdef HAVE_PTHREAD
+		pthread_attr_init(&attr);
+		pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
+
+		retcode = pthread_create(&thread, &attr, handle_socket, arg);
+		if (retcode != 0) {
 			rig_debug(RIG_DEBUG_ERR, "pthread_create: %s\n", strerror(retcode));
 			break;
 		}
+#else
+		handle_socket(arg);
+#endif
 	}
 	while (retcode == 0);
 
@@ -403,15 +445,24 @@ void * handle_socket(void *arg)
 	fsockin = fdopen(handle_data_arg->sock, "rb");
 	if (!fsockin) {
 		rig_debug(RIG_DEBUG_ERR, "fdopen in: %s\n", strerror(errno));
+		close(handle_data_arg->sock);
 		free(arg);
+#ifdef HAVE_PTHREAD
 		pthread_exit(NULL);
+#endif
+		return NULL;
 	}
 
 	fsockout = fdopen(handle_data_arg->sock, "wb");
 	if (!fsockout) {
 		rig_debug(RIG_DEBUG_ERR, "fdopen out: %s\n", strerror(errno));
+		fclose(fsockin);
+		close(handle_data_arg->sock);
 		free(arg);
+#ifdef HAVE_PTHREAD
 		pthread_exit(NULL);
+#endif
+		return NULL;
 	}
 
 	do {
@@ -430,6 +481,9 @@ void * handle_socket(void *arg)
 	close(handle_data_arg->sock);
 	free(arg);
 
+#ifdef HAVE_PTHREAD
+	pthread_exit(NULL);
+#endif
 	return NULL;
 }
 
@@ -449,11 +503,13 @@ void usage(void)
 	"  -s, --serial-speed=BAUD    set serial speed of the serial port\n"
 	"  -c, --civaddr=ID           set CI-V address, decimal (for Icom rigs only)\n"
 	"  -t, --port=NUM             set TCP listening port, default %d\n"
+	"  -T, --listen-addr=IPADDR   set listening IP address, default ANY\n"
 	"  -C, --set-conf=PARM=VAL    set config parameters\n"
 	"  -L, --show-conf            list all config parameters\n"
 	"  -l, --list                 list all model numbers and exit\n"
 	"  -u, --dump-caps            dump capabilities and exit\n"
 	"  -o, --vfo                  do not default to VFO_CURR, require extra vfo arg\n"
+	"  -e, --end-marker           use END marker in rigctld protocol\n"
 	"  -v, --verbose              set verbose mode, cumulative\n"
 	"  -h, --help                 display this help and exit\n"
 	"  -V, --version              output version information and exit\n\n",
