@@ -1,8 +1,8 @@
 /*
  *  Hamlib WiNRADiO backend - WR3100 description
- *  Copyright (c) 2001-2003 by Stephane Fillod
+ *  Copyright (c) 2001-2004 by Stephane Fillod
  *
- *	$Id: wr3100.c,v 1.4 2003/10/01 19:32:04 fillods Exp $
+ *	$Id: wr3100.c,v 1.6 2008/10/26 13:54:52 y32kn Exp $
  *
  *   This library is free software; you can redistribute it and/or modify
  *   it under the terms of the GNU Library General Public License as
@@ -20,16 +20,13 @@
  *
  */
 
-#ifdef HAVE_CONFIG_H
-#include "config.h"
-#endif
-
 #include <stdlib.h>
 
 #include <hamlib/rig.h>
 #include "winradio.h"
 
 
+#ifdef WINRADIO_IOCTL
 /*
  * Winradio rigs capabilities.
  */
@@ -47,7 +44,7 @@ const struct rig_caps wr3100_caps = {
   .mfg_name =       "Winradio",
   .version =        "0.6",
   .copyright = 	 "LGPL",
-  .status =         RIG_STATUS_NEW,
+  .status =         RIG_STATUS_UNTESTED,
   .rig_type =       RIG_TYPE_PCRECEIVER,
   .port_type =      RIG_PORT_DEVICE,
   .targetable_vfo = 	 0,
@@ -111,3 +108,4 @@ const struct rig_caps wr3100_caps = {
   .get_info =      wr_get_info,
 };
 
+#endif	/* WINRADIO_IOCTL */
