@@ -9,7 +9,7 @@
  * "CAT" interface box (FIF-232C) or similar.
  *
  *
- * $Id: ft757gx.c,v 1.5 2008/01/10 14:19:19 n0nb Exp $  
+ * $Id: ft757gx.c,v 1.10 2008/09/21 19:35:01 fillods Exp $
  *
  *
  *  This library is free software; you can redistribute it and/or
@@ -64,8 +64,8 @@ static int rig2mode(RIG *rig, int md, rmode_t *mode, pbwidth_t *width);
  *
  */
 struct ft757_priv_data {
-    unsigned char pacing;		    /* pacing value */
-    unsigned int read_update_delay;	/* depends on pacing value */
+    unsigned char pacing;           /* pacing value */
+    unsigned int read_update_delay; /* depends on pacing value */
     unsigned char current_vfo;      /* active VFO from last cmd , can be either RIG_VFO_A or RIG_VFO_B only */
     unsigned char update_data[FT757GX_STATUS_UPDATE_DATA_LENGTH]; /* returned data */
 };
@@ -79,9 +79,9 @@ const struct rig_caps ft757gx_caps = {
     .rig_model =        RIG_MODEL_FT757,
     .model_name =       "FT-757GX",
     .mfg_name =         "Yaesu",
-    .version =          "0.3.1",
+    .version =          "0.3.2",
     .copyright =        "LGPL",
-    .status =           RIG_STATUS_ALPHA,
+    .status =           RIG_STATUS_BETA,
     .rig_type =         RIG_TYPE_MOBILE,
     .ptt_type =         RIG_PTT_NONE,
     .dcd_type =         RIG_DCD_NONE,
@@ -94,8 +94,8 @@ const struct rig_caps ft757gx_caps = {
     .serial_handshake = RIG_HANDSHAKE_NONE,
     .write_delay =      FT757GX_WRITE_DELAY,
     .post_write_delay = FT757GX_POST_WRITE_DELAY,
-    .timeout =          2000,
-    .retry =            0,
+    .timeout =          FT757GX_DEFAULT_READ_TIMEOUT,
+    .retry =            10,
     .has_get_func =     RIG_FUNC_NONE,
     .has_set_func =     RIG_FUNC_NONE,
     .has_get_level =    RIG_LEVEL_NONE,
@@ -113,9 +113,9 @@ const struct rig_caps ft757gx_caps = {
     .transceive =       RIG_TRN_OFF,
     .bank_qty =         0,
     .chan_desc_sz =     0,
-    .chan_list =        { RIG_CHAN_END, },	/* FIXME: memory channel list:20 */
+    .chan_list =        { RIG_CHAN_END, },  /* FIXME: memory channel list:20 */
 
-    .rx_range_list1 =   { RIG_FRNG_END, },    /* FIXME: enter region 1 setting */
+    .rx_range_list1 =   { RIG_FRNG_END, },  /* FIXME: enter region 1 setting */
 
     .tx_range_list1 =   { RIG_FRNG_END, },
 
@@ -165,7 +165,7 @@ const struct rig_caps ft757gx_caps = {
 
     .set_freq =         ft757_set_freq,     /* set freq */
     .get_freq =         NULL,               /* get freq */
-    .set_mode =         NULL,		        /* set mode */
+    .set_mode =         NULL,               /* set mode */
     .get_mode =         NULL,               /* get mode */
     .set_vfo =          ft757_set_vfo,      /* set vfo */
 
@@ -182,11 +182,11 @@ const struct rig_caps ft757gx2_caps = {
     .rig_model =        RIG_MODEL_FT757GXII,
     .model_name =       "FT-757GXII",
     .mfg_name =         "Yaesu",
-    .version =          "0.2.1",
+    .version =          "0.3.2",
     .copyright =        "LGPL",
-    .status =           RIG_STATUS_ALPHA,
+    .status =           RIG_STATUS_STABLE,
     .rig_type =         RIG_TYPE_MOBILE,
-    .ptt_type =         RIG_PTT_SERIAL_DTR,	/* pin4? */
+    .ptt_type =         RIG_PTT_SERIAL_DTR,     /* CAT port pin 4 per manual */
     .dcd_type =         RIG_DCD_NONE,
     .port_type =        RIG_PORT_SERIAL,
     .serial_rate_min =  4800,
@@ -197,8 +197,8 @@ const struct rig_caps ft757gx2_caps = {
     .serial_handshake = RIG_HANDSHAKE_NONE,
     .write_delay =      FT757GX_WRITE_DELAY,
     .post_write_delay = FT757GX_POST_WRITE_DELAY,
-    .timeout =          2000,
-    .retry =            0,
+    .timeout =          FT757GX_DEFAULT_READ_TIMEOUT,
+    .retry =            10,
     .has_get_func =     RIG_FUNC_LOCK,
     .has_set_func =     RIG_FUNC_LOCK,
     .has_get_level =    RIG_LEVEL_RAWSTR,
@@ -261,7 +261,7 @@ const struct rig_caps ft757gx2_caps = {
     /* mode/filter list, .remember =  order matters! */
     .filters =  {
         {RIG_MODE_SSB|RIG_MODE_CW,  kHz(2.7)},
-        {RIG_MODE_CW,  Hz(600)},	/* narrow */
+        {RIG_MODE_CW,  Hz(600)},    /* narrow */
         {RIG_MODE_AM,  kHz(6)},
         {RIG_MODE_FM,  kHz(15)},
 
@@ -296,23 +296,23 @@ const struct rig_caps ft757gx2_caps = {
 int ft757_init(RIG *rig)
 {
     struct ft757_priv_data *p;
-  
+
     rig_debug(RIG_DEBUG_VERBOSE, "%s called.\n", __func__);
 
     if (!rig)
         return -RIG_EINVAL;
-  
+
     p = (struct ft757_priv_data *)malloc(sizeof(struct ft757_priv_data));
-    if (!p)			/* whoops! memory shortage! */
+    if (!p)         /* whoops! memory shortage! */
         return -RIG_ENOMEM;
-  
+
     /* TODO: read pacing from preferences */
 
-    p->pacing = FT757GX_PACING_DEFAULT_VALUE; /* set pacing to minimum for now */
+    p->pacing = FT757GX_PACING_DEFAULT_VALUE;   /* set pacing to minimum for now */
     p->read_update_delay = FT757GX_DEFAULT_READ_TIMEOUT; /* set update timeout to safe value */
-    p->current_vfo =  RIG_VFO_A;	/* default to VFO_A ? */
+    p->current_vfo =  RIG_VFO_A;                /* default to VFO_A ? */
     rig->state.priv = (void *)p;
-  
+
     return RIG_OK;
 }
 
@@ -350,7 +350,7 @@ int ft757_open(RIG *rig)
 
     if (!rig)
         return -RIG_EINVAL;
-  
+
     /* read back the 75 status bytes */
     retval = ft757_get_update_data(rig);
     if (retval < 0) {
@@ -373,11 +373,11 @@ int ft757_set_freq(RIG *rig, vfo_t vfo, freq_t freq)
 {
     unsigned char cmd[YAESU_CMD_LENGTH] = { 0x00, 0x00, 0x00, 0x00, 0x0a};
 
-    rig_debug(RIG_DEBUG_VERBOSE, "%s called.\n", __func__);
+    rig_debug(RIG_DEBUG_VERBOSE, "%s called. Freq=%"PRIfreq"\n", __func__, freq);
 
     if (!rig)
         return -RIG_EINVAL;
-  
+
     /* fill in first four bytes */
     to_bcd(cmd, freq/10, BCD_LEN);
 
@@ -420,26 +420,27 @@ int ft757_get_freq(RIG *rig, vfo_t vfo, freq_t *freq)
 
     if (!rig)
         return -RIG_EINVAL;
-  
-    retval = ft757_get_update_data(rig);	/* get whole shebang from rig */
+
+    retval = ft757_get_update_data(rig);    /* get whole shebang from rig */
     if (retval < 0)
         return retval;
 
     /* grab freq (little endian format) and convert */
     switch(vfo) {
     case RIG_VFO_CURR:
-        *freq = from_bcd(priv->update_data+STATUS_CURR_FREQ, BCD_LEN);
+        *freq = 10 * from_bcd(priv->update_data+STATUS_CURR_FREQ, BCD_LEN);
         break;
     case RIG_VFO_A:
-        *freq = from_bcd(priv->update_data+STATUS_VFOA_FREQ, BCD_LEN);
+        *freq = 10 * from_bcd(priv->update_data+STATUS_VFOA_FREQ, BCD_LEN);
         break;
     case RIG_VFO_B:
-        *freq = from_bcd(priv->update_data+STATUS_VFOB_FREQ, BCD_LEN);
+        *freq = 10 * from_bcd(priv->update_data+STATUS_VFOB_FREQ, BCD_LEN);
         break;
     default:
-        return -RIG_EINVAL;		/* sorry, wrong VFO */
+        return -RIG_EINVAL;     /* sorry, wrong VFO */
     }
 
+    rig_debug(RIG_DEBUG_VERBOSE,"%s returning: Freq=%"PRIfreq"\n", __func__, *freq );
     return RIG_OK;
 }
 
@@ -453,8 +454,8 @@ int ft757_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
 
     if (!rig)
         return -RIG_EINVAL;
-  
-    retval = ft757_get_update_data(rig);	/* get whole shebang from rig */
+
+    retval = ft757_get_update_data(rig);    /* get whole shebang from rig */
     if (retval < 0)
         return retval;
 
@@ -469,7 +470,7 @@ int ft757_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
         retval = rig2mode(rig, priv->update_data[STATUS_VFOB_MODE], mode, width);
         break;
     default:
-        return -RIG_EINVAL;		/* sorry, wrong VFO */
+        return -RIG_EINVAL;     /* sorry, wrong VFO */
     }
 
     return retval;
@@ -491,7 +492,7 @@ int ft757_set_vfo(RIG *rig, vfo_t vfo)
 
     if (!rig)
         return -RIG_EINVAL;
-  
+
     switch(vfo) {
     case RIG_VFO_CURR:
         return RIG_OK;
@@ -502,7 +503,7 @@ int ft757_set_vfo(RIG *rig, vfo_t vfo)
         cmd[3] = 0x01;          /* VFO B */
         break;
     default:
-        return -RIG_EINVAL;		/* sorry, wrong VFO */
+        return -RIG_EINVAL;     /* sorry, wrong VFO */
     }
 
     priv->current_vfo = vfo;
@@ -520,8 +521,8 @@ int ft757_get_vfo(RIG *rig, vfo_t *vfo)
 
     if (!rig)
         return -RIG_EINVAL;
-  
-    retval = ft757_get_update_data(rig);	/* get whole shebang from rig */
+
+    retval = ft757_get_update_data(rig);    /* get whole shebang from rig */
     if (retval < 0)
         return retval;
 
@@ -546,8 +547,8 @@ int ft757_get_ptt(RIG *rig, vfo_t vfo, ptt_t *ptt)
 
     if (!rig)
         return -RIG_EINVAL;
-  
-    retval = ft757_get_update_data(rig);	/* get whole shebang from rig */
+
+    retval = ft757_get_update_data(rig);    /* get whole shebang from rig */
     if (retval < 0)
         return retval;
 
@@ -565,7 +566,7 @@ int ft757_get_level(RIG *rig, vfo_t vfo, setting_t level, value_t *val)
 
     if (!rig)
         return -RIG_EINVAL;
-  
+
     if (level != RIG_LEVEL_RAWSTR)
         return -RIG_EINVAL;
 
@@ -602,27 +603,42 @@ int ft757_get_update_data(RIG *rig)
 {
     unsigned char cmd[YAESU_CMD_LENGTH] = { 0x00, 0x00, 0x00, 0x00, 0x10};
     struct ft757_priv_data *priv = (struct ft757_priv_data *)rig->state.priv;
-    int retval;
+    int retval=0;
+    int nbtries ;
+    /* Maximum number of attempts to ask/read the data. */
+    int maxtries = rig->state.rigport.retry ;
 
-    rig_debug(RIG_DEBUG_VERBOSE, "%s called.\n", __func__);
+    rig_debug(RIG_DEBUG_VERBOSE, "%s called. Timeout=%ld ms, Retry=%d\n",
+		    __func__, rig->state.rigport.timeout, maxtries);
 
-    if (!rig)
-        return -RIG_EINVAL;
-  
-    serial_flush(&rig->state.rigport);
+    /* At least on one model, returns erraticaly a timeout. Increasing the timeout
+    does not fix things. So we restart the read from scratch, it works most of times. */
+    for( nbtries = 0 ; nbtries < maxtries ; nbtries++ )
+    {
+        serial_flush(&rig->state.rigport);
 
-    /* send READ STATUS cmd to rig  */
-    retval = write_block(&rig->state.rigport, (char *)cmd, YAESU_CMD_LENGTH);
-    if (retval < 0)
-        return retval;
+        /* send READ STATUS cmd to rig  */
+        retval = write_block(&rig->state.rigport, (char *)cmd, YAESU_CMD_LENGTH);
+        if (retval < 0)
+            return retval;
 
-    /* read back the 75 status bytes */
-    retval = read_block(&rig->state.rigport,
-                        (char *)priv->update_data,
-                        FT757GX_STATUS_UPDATE_DATA_LENGTH);
+        /* read back the 75 status bytes */
+        retval = read_block(&rig->state.rigport,
+                            (char *)priv->update_data,
+                            FT757GX_STATUS_UPDATE_DATA_LENGTH);
+        if (retval == FT757GX_STATUS_UPDATE_DATA_LENGTH) {
+            break ;
+	}
+        rig_debug(RIG_DEBUG_ERR,
+                  "%s: read update_data failed, %d octets of %d read, retry %d out of %d\n",
+                  __func__, retval, FT757GX_STATUS_UPDATE_DATA_LENGTH,
+	          nbtries, maxtries);
+	/* The delay is quadratic. */
+	sleep(nbtries*nbtries);
+    }
 
     if (retval != FT757GX_STATUS_UPDATE_DATA_LENGTH) {
-        rig_debug(RIG_DEBUG_ERR,"%s: read update_data failed, %d octects of %d read.\n",
+        rig_debug(RIG_DEBUG_ERR,"%s: read update_data failed, %d octets of %d read.\n",
                   __func__, retval, FT757GX_STATUS_UPDATE_DATA_LENGTH);
 
         return retval < 0 ? retval : -RIG_EIO;
@@ -640,7 +656,7 @@ int mode2rig(RIG *rig, rmode_t mode, pbwidth_t width)
 
     if (!rig)
         return -RIG_EINVAL;
-  
+
     /*
      * translate mode from generic to ft757 specific
      */
@@ -678,7 +694,7 @@ int rig2mode(RIG *rig, int md, rmode_t *mode, pbwidth_t *width)
 
     if (!rig)
         return -RIG_EINVAL;
-  
+
     /*
      * translate mode from ft757 specific to generic
      */
