@@ -13,7 +13,7 @@
  * The starting point for this code was Frank's ft847 implementation.
  *
  *
- *    $Id: ft857.c,v 1.9 2006/10/07 15:51:38 csete Exp $  
+ *    $Id: ft857.c,v 1.11 2008/10/31 22:09:11 fillods Exp $  
  *
  *
  *  This library is free software; you can redistribute it and/or
@@ -130,9 +130,9 @@ static const yaesu_cmd_set_t ncmd[] = {
 };
 
 
-#define FT857_ALL_RX_MODES      (RIG_MODE_AM|RIG_MODE_CW|RIG_MODE_USB|\
+#define FT857_ALL_RX_MODES      (RIG_MODE_AM|RIG_MODE_CW|RIG_MODE_CWR|RIG_MODE_USB|\
                                  RIG_MODE_LSB|RIG_MODE_RTTY|RIG_MODE_FM)
-#define FT857_SSB_CW_RX_MODES   (RIG_MODE_CW|RIG_MODE_USB|RIG_MODE_LSB)
+#define FT857_SSB_CW_RX_MODES   (RIG_MODE_CW|RIG_MODE_CWR|RIG_MODE_USB|RIG_MODE_LSB)
 #define FT857_AM_FM_RX_MODES    (RIG_MODE_AM|RIG_MODE_FM)
 
 #define FT857_OTHER_TX_MODES    (RIG_MODE_AM|RIG_MODE_CW|RIG_MODE_USB|\
@@ -146,9 +146,9 @@ const struct rig_caps ft857_caps = {
   .rig_model = 		RIG_MODEL_FT857,
   .model_name = 	"FT-857", 
   .mfg_name = 		"Yaesu", 
-  .version = 		"0.2", 
+  .version = 		"0.2.2",
   .copyright = 		"LGPL",
-  .status = 		RIG_STATUS_ALPHA,
+  .status = 		RIG_STATUS_BETA,
   .rig_type = 		RIG_TYPE_TRANSCEIVER,
   .ptt_type = 		RIG_PTT_RIG,
   .dcd_type = 		RIG_DCD_RIG,
@@ -234,7 +234,12 @@ const struct rig_caps ft857_caps = {
   },  
 
   .filters =  {
-    RIG_FLT_END,
+	{RIG_MODE_SSB|RIG_MODE_CW|RIG_MODE_CWR|RIG_MODE_RTTY, kHz(2.2)},
+	{RIG_MODE_AM, kHz(6)},
+	{RIG_MODE_FM|RIG_MODE_PKTFM, kHz(15)},
+	{RIG_MODE_FM|RIG_MODE_PKTFM, kHz(9)},
+	{RIG_MODE_WFM, kHz(230)},	/* ?? */
+	RIG_FLT_END,
   },
 
   .priv = 		NULL,
@@ -448,9 +453,11 @@ int ft857_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
     *mode = RIG_MODE_USB;
     break;
   case 0x02:
+  case 0x82:
     *mode = RIG_MODE_CW;
     break;
   case 0x03:
+  case 0x83:
     *mode = RIG_MODE_CWR;
     break;
   case 0x04:
@@ -460,11 +467,17 @@ int ft857_get_mode(RIG *rig, vfo_t vfo, rmode_t *mode, pbwidth_t *width)
     *mode = RIG_MODE_WFM;
     break;
   case 0x08:
+  case 0x88:
     *mode = RIG_MODE_FM;
     break;
   case 0x0a:
+  case 0x8a:
     *mode = RIG_MODE_RTTY;
     break;
+  case 0x0c:
+  case 0x8c:
+    *mode = RIG_MODE_PKTFM;
+    break;
   default:
     *mode = RIG_MODE_NONE;
   }
@@ -675,6 +688,9 @@ int ft857_set_mode(RIG *rig, vfo_t vfo, rmode_t mode, pbwidth_t width)
   case RIG_MODE_CWR:
     index = FT857_NATIVE_CAT_SET_MODE_CWR;
     break;
+  case RIG_MODE_PKTFM:
+    index = FT857_NATIVE_CAT_SET_MODE_PKT;
+    break;
   default:
     return -RIG_EINVAL;
   }
