@@ -7,7 +7,7 @@
  * via serial interface to a Yaesu rig
  *
  *
- *	$Id: yaesu.c,v 1.27 2007/11/23 03:31:26 n0nb Exp $  
+ *	$Id: yaesu.c,v 1.29 2008/11/01 16:38:46 fillods Exp $  
  *
  *
  *  This library is free software; you can redistribute it and/or
@@ -89,6 +89,7 @@ DECLARE_INITRIG_BACKEND(yaesu)
   rig_register(&ft890_caps);
   rig_register(&ft900_caps);
   rig_register(&ft920_caps);
+  rig_register(&ft950_caps);
   rig_register(&ft990_caps);
   rig_register(&ft1000d_caps);
   rig_register(&ft1000mp_caps);
@@ -112,7 +113,7 @@ DECLARE_INITRIG_BACKEND(yaesu)
  */
 DECLARE_PROBERIG_BACKEND(yaesu)
 {
-	unsigned char idbuf[YAESU_CMD_LENGTH];
+	unsigned char idbuf[YAESU_CMD_LENGTH+1];
 	static const unsigned char cmd[YAESU_CMD_LENGTH] = { 0x00, 0x00, 0x00, 0x00, 0xfa};
 	int id_len=-1, i, id1, id2;
 	int retval=-1;
@@ -157,7 +158,7 @@ DECLARE_PROBERIG_BACKEND(yaesu)
 	 * reply should be [Flag1,Flag2,Flag3,ID1,ID2]
 	 */
 	if (id_len != 5 || id_len != 6) {
-		idbuf[7] = '\0';
+		idbuf[YAESU_CMD_LENGTH] = '\0';
 		rig_debug(RIG_DEBUG_WARN,"probe_yaesu: protocol error,"
 			" expected %d, received %d: %s\n", 6, id_len, idbuf);
 		return RIG_MODEL_NONE;
